/* $Header: /src/carson/nistnet/RCS/nistnet.h,v 1.6 2000/03/22 16:06:26 carson Exp carson $ */

#ifndef _NISTNET_H
#define _NISTNET_H

/* nistnet.h - Linux implementation of "hitbox"-like functionality */

/* structure used for requesting the nistnet hitbox */
struct lin_hitreq {
	u_int32_t	src;		/* source/destination to hit */
	u_int32_t	dest;		/* either may be set to INADDR_ANY */
	unsigned int p_drop;	/* probability of drop times 65536 */
	unsigned int p_dup;	/* probability of duplicate times 65536 */
	unsigned int delay;	/* mean delay on packets in microseconds */
	unsigned int delsigma;	/* standard deviation on delay in usecs */
	unsigned int bandwidth;	/* max bandwidth (bytes/sec) on this pair */

	unsigned long qsize;	/* shouldn't be here ... */
	/* In order to keep the structure the same size, I'm dropping
	 * the unused "queue_mgr" field and replacing it with an
	 * explicit congestion notification threshhold value
	 */
	/*int	queue_mgr;*/	/* not used */
/* We actually implement the simpler DRD algorithm rather than RED,
 * but for our purposes they're close enough that these defines seem ok...
 *	Constraints: drd_min <= drd_congestion <= drd_max unless
 *	drd_congestion == 0.
 */
#ifdef CONFIG_ECN
	/* I decided not to make the structure change conditional on
	 * CONFIG_ECN, since there wasn't any point in retaining the
	 * unused field.
	 */
#endif
	int	drd_congestion;	/* DRD "congestion enabled" threshhold */
	int	drd_max;	/* DRD max queue length */
#define red_th_max	drd_max
	int	drd_min;	/* DRD min queue length */
#define red_th_min	drd_min
};

#ifdef CONFIG_ECN

/* I haven't seen standard defines for the ECN bits yet, so will add them
 * here for now.  They are in the tos byte in the IP header.
 */
#ifndef _ECN_
#define _ECN_

#define	ECN_CAPABLE	2
#define ECN_NOTED	1

#endif	/* _ECN_ */

#endif /* CONFIG_ECN */

#define MILLION	1000000		/* just so I don't do the wrong no. of 0's */
#define PROBFACTOR	65536	/* All probabilities are scaled by this */


/* structure used for returning stats on the hitbox */

struct lin_hitstats {
	struct lin_hitreq hitreq;
	int	n_drops;
	int	rand_drops;	/* random drops */
	int	mem_drops;	/* out of memory errors */
	int	drd_drops;	/* DRD algorithm drops */
#define	red_drops	drd_drops
	int	dups;
/* Statistics used in manipulating bandwidith */
	struct timeval next_packet; /* when next packet can be sent */
	unsigned long	current_size;	/* size of packet being sent currently */
	struct timeval last_packet; /* when last packet was sent */
	unsigned long	current_bandwidth;	/* Approximation of bandwidth used */
	unsigned long bytes_sent;
	unsigned long qlen;
#define BAND_ARRAY	10	/* bandwidth averaged over 10 seconds */
	unsigned long bandwidth_array[BAND_ARRAY];
	int seats_used;
#ifdef CONFIG_ECN
	/* Again, to keep the size consistent, the inclusion of this
	 * field is made unconditional; if ECN is not used, this will
	 * be always 0.
	 */
#endif
	int	drd_ecns;	/* DRD explicit congestion notifies */
/* None of these others are currently used */
	int	if_used;
	int	connection_count;
	int	gateway_mode;
	unsigned int mtu;
	int	red_save_arg;
	int	reset;
	/*int	virtual_timer; drop to keep structure size the same */
};

struct lin_globalstats {	/* Global emulator statistics */
	unsigned long  process_overhead[BAND_ARRAY];
	unsigned long  unprocess_overhead[BAND_ARRAY];
	unsigned long  hash_tries[BAND_ARRAY];
};

/* This is a temporary bridge to the new code... */
struct lin_nglobalstats {	/* New global emulator statistics */
	struct lin_globalstats	l;
	unsigned int emulator_on;	/* 1 if on, 0 if off */
};

struct addrpair {
	u_int32_t	src;		/* source/destination to hit */
	u_int32_t	dest;		/* either may be set to INADDR_ANY */
};

/* We decide which net headers to include based on whether we're
 * using glibc or not...
 */
#if defined(__GLIBC__) && !defined(__KERNEL__)
/*#include <net/protocol.h>*/
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>
#include <netinet/igmp.h>
#else
#include <linux/netdevice.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/icmp.h>
#ifdef __KERNEL__
#include <net/protocol.h>
#endif
#include <linux/igmp.h>
#endif

struct lin_mungestats {
	struct iphdr ip;
	union {
		struct tcphdr tcp;
		struct udphdr udp;
		struct icmphdr icmp;
		struct igmphdr igmp;
	} h;
};
/* ioctl's */

/* hitbox */
#define HITIOCTL_OFF	_IO('h', 0)	/* master switch */
#define HITIOCTL_ON	_IO('h', 1)
#define HITIOCTL_ADD	_IOW('h', 2, struct lin_hitreq)
#define HITIOCTL_REMOVE	_IOW('h', 3, struct lin_hitreq)
#define HITIOCTL_STATS	_IOW('h', 4, struct lin_hitstats)
#define HITIOCTL_MODE	_IOW('h', 5, int)
#define HITIOCTL_TIMER	_IOW('h', 6, int)
#define HITIOCTL_MTU	_IOW('h', 7, int)
#define HITIOCTL_DEBUG	_IOW('h', 8, int)
#define HITIOCTL_GLOBALSTATS	_IOW('h', 9, struct lin_globalstats)
#define HITIOCTL_NGLOBALSTATS	_IOW('h', 10, struct lin_nglobalstats)

/* mungebox */
#define MUNGEIOCTL_OFF		_IO('m', 0)	/* master switch */
#define MUNGEIOCTL_ON		_IO('m', 1)
#define MUNGEIOCTL_WAIT		_IO('m', 2)	/* hold packets */
#define MUNGEIOCTL_NOWAIT	_IO('m', 3)	/* don't hold */
#define MUNGEIOCTL_SET		_IOW('m', 4, struct addrpair) /* set monitored pair */
#define MUNGEIOCTL_UNSET	_IO('m', 5) /* unset monitored pair */
#define MUNGEIOCTL_STAT		_IOW('m', 6, struct lin_mungestats)
#define MUNGEIOCTL_DEBUG	_IOW('m', 7, int)
/* also FIOASYNC on/off */

/* Kernel stuff */
#ifdef __KERNEL__
/* Hash table for hitbox entries */
struct lin_hitbox {
	struct lin_hitstats stats;
#define hit_entry	stats.hitreq
	struct lin_hitbox *next;
};

struct hit_packetinfo {
	struct sk_buff *skb;
	struct device *dev;
	struct lin_hitbox *hitme;
	unsigned long flags;
};

typedef int (*packet_munger)(struct sk_buff *, struct device *,
	struct packet_type *, struct lin_hitbox *);

extern void addmunge(packet_munger munger);
extern void rmmunge(packet_munger munger);

struct lin_mungebox {
	struct addrpair munge_who;
	unsigned int	munge_flags;		/* only wait-related right now */
#define MUNGE_WAIT	0x1			/* packet should wait */
#define MUNGE_UPDATED	0x2			/* new data available */
	struct wait_queue *reader_wait;
	struct wait_queue *packet_wait;
	struct lin_mungestats munge_stats;
	struct lin_mungebox *next;
};
#endif

#endif /* _NISTNET_H */
