/* $Header: /src/carson/nistnet/RCS/tabledist.c,v 1.3 2000/03/22 16:06:26 carson Exp carson $ */

/* tabledist.c - distribution table used to convert a uniformly distributed
 * pseudo-random variable into some other desired distribution.  By default,
 * a pareto-normal distribution which resembles real observed packet delay
 * characteristics is used.
 */
#ifdef __KERNEL__

#include "kincludes.h"

#else

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#endif

#include "tabledist.h"

#if defined(USE_PARETONORMAL)
#include "paretonormal.h"
#elif defined(USE_PARETO)
#include "pareto.h"
#else /* USE_NORMAL */
#include "normal.h"
#endif

#ifdef __KERNEL__

/* very bad, but placeholder for now */
long
random(void)
{
	static int dirty=2;

	dirty *= 69069;
	dirty += 85;
	return dirty;
}
#endif

/* Table manipulation */

int
tabledistsize(void)
{
	return sizeof(disttable);
}

void
tabledistfill(const char *buf)
{
#ifdef __KERNEL__
	memcpy_fromfs(disttable, (short int *)buf, (unsigned long) tabledistsize());
#else
	memcpy((void *)disttable, (void *)buf, tabledistsize());
#endif
}

/* Return a pseudo-randomly distributed value with mean mu and std deviation
 * sigma.  Uses table lookup to approximate the desired distribution, and a
 * uniformly-distributed pseudo-random source.
 */

int 
tabledist(int mu, int sigma)
{
	int x;
	int index;
	int sigmamod, sigmadiv;	/* watch for overflow!! */

	if (sigma == 0)
		return mu;

	index = (random() & (TABLESIZE-1));
	sigmamod = sigma%TABLEFACTOR;
	sigmadiv = sigma/TABLEFACTOR;
	x = sigmamod*disttable[index];

	if (x >= 0)
		x += TABLEFACTOR/2;
	else
		x -= TABLEFACTOR/2;

	x /= TABLEFACTOR;
	x += sigmadiv*disttable[index];
	x += mu;
	return x;
}
